@@ -1,3 +1,6 @@
+"""
+Core of Logux Django
+"""
 from __future__ import annotations
 
 import json
@@ -6,7 +9,7 @@ import re
 from abc import abstractmethod, ABC
 from copy import deepcopy
 from datetime import datetime
-from typing import List, Callable, Optional, Dict, NoReturn, NewType, Union, Sequence, Any
+from typing import List, Callable, Optional, Dict, NewType, Union, Sequence, Any
 
 import requests
 from django.conf import settings
@@ -33,7 +36,7 @@ def protocol_version_is_supported(version: int) -> bool:
     return version == LOGUX_PROTOCOL_VERSION
 
 
-class Meta:
+class Meta:  # pylint: disable=too-many-instance-attributes
     """ Logux meta: https://logux.io/guide/concepts/meta/
     TODO: add docs about comp:
       https://github.com/logux/django/issues/12#issuecomment-612394901
@@ -55,6 +58,8 @@ class Meta:
         self.node_id: Optional[str] = self._get_node_id()
         self.time: datetime = self._get_time()
 
+        self.subprotocol: str = self._get_subprotocol()
+
     def __getitem__(self, item):
         return self._raw_meta[item]
 
@@ -65,9 +70,13 @@ class Meta:
         return not self.__eq__(o)
 
     def __lt__(self, other: Meta) -> bool:
+        # pylint: disable=no-else-return,too-many-return-statements
         # <
-        if self.get_raw_meta() and not other.get_raw_meta():
-            return False
+        if self.get_raw_meta():
+            if not other.get_raw_meta():
+                return False
+            elif not self.get_raw_meta() and other.get_raw_meta():
+                return True
         elif not self.get_raw_meta() and other.get_raw_meta():
             return True
 
@@ -89,6 +98,7 @@ class Meta:
         return False
 
     def __gt__(self, other: Meta) -> bool:
+        # pylint: disable=no-else-return,too-many-return-statements
         # >
         if self.get_raw_meta() and not other.get_raw_meta():
             return True
@@ -158,10 +168,15 @@ class Meta:
          """
         return datetime.fromtimestamp(int(self.id.split(' ')[0]) / 1e3)
 
+    def _get_subprotocol(self):
+        return self._raw_meta.get('subprotocol')
+
     def get_raw_meta(self) -> Dict:
+        """ Get the copy of raw meta dict """
         return deepcopy(self._raw_meta)
 
     def get_json(self) -> str:
+        """ Get raw meta and convert it to JSON """
         return json.dumps(self._raw_meta)
 
 
@@ -180,7 +195,7 @@ def logux_add(action: Action, raw_meta: Optional[Dict] = None) -> None:
     TODO: extract this exception into custom error class -> logux/exception.py
     :raises: base Exception() if Logux Proxy returns non 200 response code
 
-    :return: NoReturn
+    :return: None
     """
     command = {
         "version": LOGUX_PROTOCOL_VERSION,
@@ -194,13 +209,13 @@ def logux_add(action: Action, raw_meta: Optional[Dict] = None) -> None:
         ]
     }
 
-    logger.debug(f'logux_add action {action} with meta {raw_meta or {}} to Logux')
+    logger.debug('logux_add action %s with meta %s to Logux', action, raw_meta or {})
 
     r = requests.post(url=settings.LOGUX_URL, json=command)
-    logger.debug(f'Logux answer is {r.status_code}: {r.text}')
+    logger.debug('Logux answer is %s: %s', r.status_code, r.text)
 
     if r.status_code != 200:
-        logger.error(f'`logux_add` to Logux is failed! err: {r.status_code}: {r.text}')
+        logger.error('`logux_add` to Logux is failed! err: %s: %s', r.status_code, r.text)
         raise LoguxProxyException(f'Non 200 response from Logux Proxy (logux_add): {r.status_code}: {r.text}')
 
 
@@ -251,7 +266,7 @@ class AuthCommand(Command):
         :returns:  `authenticated` or `denied` action dependently if user is authenticated.
         """
         is_authenticated: bool = self.logux_auth(self.user_id, self.token)
-        logger.warning(f'user: {self.user_id} is not authenticated')
+        logger.warning('user: %s is not authenticated', self.user_id)
         auth_id: str = self.auth_id
         return [LoguxValue(['authenticated' if is_authenticated else 'denied', auth_id])]
 
@@ -263,7 +278,7 @@ class ActionCommand(Command):
     #    DefaultActionDispatcher will raise ValueError('`action_type` attribute is required for all Actions') Exception
     action_type: str
 
-    def __init__(self, cmd_body: List[Action]):
+    def __init__(self, cmd_body: List):
         """ Construct Action cmd from raw logux command.
 
         :param cmd_body: raw logux cmd, like:
@@ -279,12 +294,12 @@ class ActionCommand(Command):
 
     @property
     def action(self):
-        # do not change internal action state from outside
+        """ Get copy of Action. Do not change internal action state from outside. """
         return deepcopy(self._action)
 
     @property
     def meta(self):
-        # do not change internal meta state from outside
+        """ Get copy of Meta. Do not change internal meta state from outside. """
         return deepcopy(self._meta)
 
     def send_back(self, action: Action, raw_meta: Optional[Dict] = None) -> None:
@@ -331,7 +346,7 @@ class ActionCommand(Command):
         logux_add(undo_action, undo_meta)
 
     # Required and optional action methods (these methods should be implemented by consumer)
-    def _finally(self, action: Action, meta: Meta) -> LoguxValue:  # type: ignore
+    def _finally(self, action: Action, meta: Meta) -> LoguxValue:  # pylint: disable=unused-argument,no-self-use
         """ Callback which will be run on the end of action/subscription processing or on an error """
         return LoguxValue([])
 
@@ -348,10 +363,10 @@ class ActionCommand(Command):
         """
         raise NotImplementedError()
 
-    def resend(self, action: Action, meta: Optional[Meta]) -> Dict:
+    def resend(self, action: Action, meta: Optional[Meta]) -> Dict:  # pylint: disable=unused-argument,no-self-use
         """ `resend` should return recipients for this action.
         It should look like:
-            {'channels': ['users/38']}
+        {'channels': ['users/38']}
         and may content fields: channels, users, nodes, clients.
 
         For more information: https://logux.io/node-api/#resend
@@ -387,7 +402,7 @@ class ActionCommand(Command):
         # access
         try:
             access_result = ['approved' if self.access(self._action, self._meta) else 'denied', self._meta.id]
-        except Exception as access_err:
+        except Exception as access_err:  # pylint: disable=broad-except
             access_result = ['error', self._meta.id, f'{access_err}']
 
         applying_result.append(access_result)
@@ -397,7 +412,7 @@ class ActionCommand(Command):
             try:
                 self.process(self._action, self._meta)
                 process_result = ['processed', self._meta.id]
-            except Exception as process_err:
+            except Exception as process_err:  # pylint: disable=broad-except
                 process_result = ['error', self._meta.id, f'{process_err}']
 
             applying_result.append(process_result)
@@ -406,7 +421,7 @@ class ActionCommand(Command):
         try:
             self._finally(self._action, self._meta)
             finally_result = []
-        except Exception as finally_err:
+        except Exception as finally_err:  # pylint: disable=broad-except
             finally_result = ['error', self._meta.id, f'{finally_err}']
 
         applying_result.append(finally_result)
@@ -422,11 +437,11 @@ class ChannelCommand(ActionCommand):
     For more information: https://logux.io/protocols/backend/examples/#subscription
 
     Subscription actions should look like:
-        [
-          "action",
-          { type: 'logux/subscribe', channel: '38/name' },
-          { id: "1560954012858 38:Y7bysd:O0ETfc 0", time: 1560954012858 }
-        ]
+    [
+        "action",
+        { type: 'logux/subscribe', channel: '38/name' },
+        { id: "1560954012858 38:Y7bysd:O0ETfc 0", time: 1560954012858 }
+    ]
     """
     # `channel_pattern` is required property, if property does not defined
     #   DefaultSubscriptionsDispatcher will raise
@@ -445,11 +460,12 @@ class ChannelCommand(ActionCommand):
 
     @classmethod
     def is_match(cls, channel: str) -> bool:
-        return True if re.match(cls.channel_pattern, channel) is not None else False  # type: ignore
+        """ Check if the Dispatcher contains channel handler """
+        return re.match(cls.channel_pattern, channel) is not None  # type: ignore
 
     # Required and optional action methods (these methods should be implemented by consumer)
     @abstractmethod
-    def load(self, action: Action, meta: Meta) -> NoReturn:
+    def load(self, action: Action, meta: Meta) -> None:
         """ `load` should contain consumer code for applying subscription.
         Generally this method is almost the same as `process`. If it raised exception,
         self.apply will return error action automatically. If `load` return error action
@@ -468,7 +484,7 @@ class ChannelCommand(ActionCommand):
         # access
         try:
             access_result = ['approved' if self.access(self._action, self._meta) else 'denied', self._meta.id]
-        except Exception as access_err:
+        except Exception as access_err:  # pylint: disable=broad-except
             access_result = ['error', self._meta.id, f'{access_err}']
 
         # load
@@ -476,7 +492,7 @@ class ChannelCommand(ActionCommand):
             try:
                 self.load(self._action, self._meta)
                 load_result = ['processed', self._meta.id]
-            except Exception as load_err:
+            except Exception as load_err:  # pylint: disable=broad-except
                 load_result = ['error', self._meta.id, f'{load_err}']
 
             applying_result.append(LoguxValue(load_result))
